<?php
namespace MM\MmDynbanner\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class PublisherController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * publisherRepository
	 *
	 * @var \MM\MmDynbanner\Domain\Repository\PublisherRepository
	 * @inject
	 */
	protected $publisherRepository;

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$publishers = $this->publisherRepository->findAll();
		$this->view->assign('publishers', $publishers);
	}

	/**
	 * action show
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $publisher
	 * @return void
	 */
	public function showAction(\MM\MmDynbanner\Domain\Model\Publisher $publisher) {
		$this->view->assign('publisher', $publisher);
	}

	/**
	 * action new
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $newPublisher
	 * @dontvalidate $newPublisher
	 * @return void
	 */
	public function newAction(\MM\MmDynbanner\Domain\Model\Publisher $newPublisher = NULL) {
		if ($newPublisher == NULL) { // workaround for fluid bug ##5636
			$newPublisher = t3lib_div::makeInstance('');
		}
		$this->view->assign('newPublisher', $newPublisher);
	}

	/**
	 * action create
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $newPublisher
	 * @return void
	 */
	public function createAction(\MM\MmDynbanner\Domain\Model\Publisher $newPublisher) {
		$this->publisherRepository->add($newPublisher);
		$this->redirect('list');
	}

	/**
	 * action edit
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $publisher
	 * @return void
	 */
	public function editAction(\MM\MmDynbanner\Domain\Model\Publisher $publisher) {
		$this->view->assign('publisher', $publisher);
	}

	/**
	 * action update
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $publisher
	 * @return void
	 */
	public function updateAction(\MM\MmDynbanner\Domain\Model\Publisher $publisher) {
		$this->publisherRepository->update($publisher);
		$this->redirect('list');
	}

	/**
	 * action delete
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Publisher $publisher
	 * @return void
	 */
	public function deleteAction(\MM\MmDynbanner\Domain\Model\Publisher $publisher) {
		$this->publisherRepository->remove($publisher);
		$this->redirect('list');
	}

}
?>