<?php
namespace MM\MmDynbanner\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class RateController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * rateRepository
	 *
	 * @var \MM\MmDynbanner\Domain\Repository\RateRepository
	 * @inject
	 */
	protected $rateRepository;

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
        $args = $this->request->getArguments();
        $filter = $args['filter']['searchString'];
        $archived = intval($args['filter']['archived']);
		if ($archived && $filter) {
		    $allrates = $this->rateRepository->findAll();
		    $rates = $this->rateRepository->findByFilter($filter);
		} elseif ($archived && !$filter) {
		    $allrates = $this->rateRepository->findAll();
		    $rates = $this->rateRepository->findAll();
		} elseif (!$archived && $filter) {
		    $allrates = $this->rateRepository->findByArchived(0);
		    $rates = $this->rateRepository->findByFilterArchived($filter);
		} else {
		    $allrates = $this->rateRepository->findByArchived(false);
		    $rates = $this->rateRepository->findByArchived(false);
		}

		$this->view->assign('filter', $args['filter']);
		$this->view->assign('rates', $rates);
		$this->view->assign('allrates', $allrates);
	}

	/**
	 * action show
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $rate
	 * @return void
	 */
	public function showAction(\MM\MmDynbanner\Domain\Model\Rate $rate) {
		$this->view->assign('rate', $rate);
	}

	/**
	 * action new
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $newRate
	 * @dontvalidate $newRate
	 * @return void
	 */
	public function newAction(\MM\MmDynbanner\Domain\Model\Rate $newRate = NULL) {
		$this->view->assign('newRate', $newRate);
	}

	/**
	 * action create
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $newRate
	 * @return void
	 */
	public function createAction(\MM\MmDynbanner\Domain\Model\Rate $newRate) {
		$this->rateRepository->add($newRate);
		$this->redirect('list');
	}

	/**
	 * action edit
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $rate
	 * @return void
	 */
	public function editAction(\MM\MmDynbanner\Domain\Model\Rate $rate) {
		$this->view->assign('rate', $rate);
	}

	/**
	 * action update
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $rate
	 * @return void
	 */
	public function updateAction(\MM\MmDynbanner\Domain\Model\Rate $rate) {
		$this->rateRepository->update($rate);
		$this->redirect('list');
	}

	/**
	 * action delete
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $rate
	 * @return void
	 */
	public function deleteAction(\MM\MmDynbanner\Domain\Model\Rate $rate) {
		$this->rateRepository->remove($rate);
		$this->redirect('list');
	}

}
?>