<?php
namespace MM\MmDynbanner\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class DeviceController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * deviceRepository
	 *
	 * @var \MM\MmDynbanner\Domain\Repository\DeviceRepository
	 * @inject
	 */
	protected $deviceRepository;

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$devices = $this->deviceRepository->findAll();
		$this->view->assign('devices', $devices);
	}

	/**
	 * action show
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $device
	 * @return void
	 */
	public function showAction(\MM\MmDynbanner\Domain\Model\Device $device) {
		$this->view->assign('device', $device);
	}

	/**
	 * action new
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $newDevice
	 * @dontvalidate $newDevice
	 * @return void
	 */
	public function newAction(\MM\MmDynbanner\Domain\Model\Device $newDevice = NULL) {
		if ($newDevice == NULL) { // workaround for fluid bug ##5636
			$newDevice = t3lib_div::makeInstance('');
		}
		$this->view->assign('newDevice', $newDevice);
	}

	/**
	 * action create
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $newDevice
	 * @return void
	 */
	public function createAction(\MM\MmDynbanner\Domain\Model\Device $newDevice) {
		$this->deviceRepository->add($newDevice);
		$this->flashMessageContainer->add('Your new Device was created.');
		$this->redirect('list');
	}

	/**
	 * action edit
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $device
	 * @return void
	 */
	public function editAction(\MM\MmDynbanner\Domain\Model\Device $device) {
		$this->view->assign('device', $device);
	}

	/**
	 * action update
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $device
	 * @return void
	 */
	public function updateAction(\MM\MmDynbanner\Domain\Model\Device $device) {
		$this->deviceRepository->update($device);
		$this->flashMessageContainer->add('Your Device was updated.');
		$this->redirect('list');
	}

	/**
	 * action delete
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $device
	 * @return void
	 */
	public function deleteAction(\MM\MmDynbanner\Domain\Model\Device $device) {
		$this->deviceRepository->remove($device);
		$this->flashMessageContainer->add('Your Device was removed.');
		$this->redirect('list');
	}

}
?>