<?php
namespace MM\MmDynbanner\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Option extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * Name
	 *
	 * @var \string
	 * @validate NotEmpty
	 */
	protected $name;

	/**
	 * Available
	 *
	 * @var boolean
	 */
	protected $available = FALSE;

	/**
	 * Manufacturer
	 *
	 * @var \string
	 */
	protected $manufacturer;

	/**
	 * Headline
	 *
	 * @var \string
	 */
	protected $headline;

	/**
	 * HC Size
	 *
	 * @var \integer
	 */
	protected $hcSize;

	/**
	 * Subline
	 *
	 * @var \string
	 */
	protected $subline;

	/**
	 * Eyecatcher Start
	 *
	 * @var \string
	 */
	protected $eyecStart;

	/**
	 * Eyecatcher Text
	 *
	 * @var \string
	 */
	protected $eyecText;

	/**
	 * Phase 1
	 *
	 * @var \float
	 */
	protected $phase1;

	/**
	 * Phase 2
	 *
	 * @var \float
	 */
	protected $phase2;

	/**
	 * Phase 3
	 *
	 * @var \float
	 */
	protected $phase3;

	/**
	 * Legal
	 *
	 * @var \string
	 */
	protected $legal;

	/**
	 * Image
	 *
	 * @var \string
	 */
	protected $image;

	/**
	 * PKID
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Pkid
	 * @lazy
	 */
	protected $pkid;

	/**
	 * Returns the name
	 *
	 * @return \string $name
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * Sets the name
	 *
	 * @param \string $name
	 * @return void
	 */
	public function setName($name) {
		$this->name = $name;
	}

	/**
	 * Returns the available
	 *
	 * @return boolean $available
	 */
	public function getAvailable() {
		return $this->available;
	}

	/**
	 * Sets the available
	 *
	 * @param boolean $available
	 * @return void
	 */
	public function setAvailable($available) {
		$this->available = $available;
	}

	/**
	 * Returns the boolean state of available
	 *
	 * @return boolean
	 */
	public function isAvailable() {
		return $this->getAvailable();
	}

	/**
	 * Returns the manufacturer
	 *
	 * @return \string $manufacturer
	 */
	public function getManufacturer() {
		return $this->manufacturer;
	}

	/**
	 * Sets the manufacturer
	 *
	 * @param \string $manufacturer
	 * @return void
	 */
	public function setManufacturer($manufacturer) {
		$this->manufacturer = $manufacturer;
	}

	/**
	 * Returns the headline
	 *
	 * @return \string $headline
	 */
	public function getHeadline() {
		return $this->headline;
	}

	/**
	 * Sets the headline
	 *
	 * @param \string $headline
	 * @return void
	 */
	public function setHeadline($headline) {
		$this->headline = $headline;
	}

	/**
	 * Returns the hcSize
	 *
	 * @return \integer $hcSize
	 */
	public function getHcSize() {
		return $this->hcSize;
	}

	/**
	 * Sets the hcSize
	 *
	 * @param \integer $hcSize
	 * @return void
	 */
	public function setHcSize($hcSize) {
		$this->hcSize = $hcSize;
	}

	/**
	 * Returns the subline
	 *
	 * @return \string $subline
	 */
	public function getSubline() {
		return $this->subline;
	}

	/**
	 * Sets the subline
	 *
	 * @param \string $subline
	 * @return void
	 */
	public function setSubline($subline) {
		$this->subline = $subline;
	}

	/**
	 * Returns the eyecStart
	 *
	 * @return \string $eyecStart
	 */
	public function getEyecStart() {
		return $this->eyecStart;
	}

	/**
	 * Sets the eyecStart
	 *
	 * @param \string $eyecStart
	 * @return void
	 */
	public function setEyecStart($eyecStart) {
		$this->eyecStart = $eyecStart;
	}

	/**
	 * Returns the eyecText
	 *
	 * @return \string $eyecText
	 */
	public function getEyecText() {
		return $this->eyecText;
	}

	/**
	 * Sets the eyecText
	 *
	 * @param \string $eyecText
	 * @return void
	 */
	public function setEyecText($eyecText) {
		$this->eyecText = $eyecText;
	}

	/**
	 * Returns the phase1
	 *
	 * @return \float $phase1
	 */
	public function getPhase1() {
		return $this->phase1;
	}

	/**
	 * Sets the phase1
	 *
	 * @param \float $phase1
	 * @return void
	 */
	public function setPhase1($phase1) {
		$this->phase1 = $phase1;
	}

	/**
	 * Returns the phase2
	 *
	 * @return \float $phase2
	 */
	public function getPhase2() {
		return $this->phase2;
	}

	/**
	 * Sets the phase2
	 *
	 * @param \float $phase2
	 * @return void
	 */
	public function setPhase2($phase2) {
		$this->phase2 = $phase2;
	}

	/**
	 * Returns the phase3
	 *
	 * @return \float $phase3
	 */
	public function getPhase3() {
		return $this->phase3;
	}

	/**
	 * Sets the phase3
	 *
	 * @param \float $phase3
	 * @return void
	 */
	public function setPhase3($phase3) {
		$this->phase3 = $phase3;
	}

	/**
	 * Returns the legal
	 *
	 * @return \string $legal
	 */
	public function getLegal() {
		return $this->legal;
	}

	/**
	 * Sets the legal
	 *
	 * @param \string $legal
	 * @return void
	 */
	public function setLegal($legal) {
		$this->legal = $legal;
	}

	/**
	 * Returns the image
	 *
	 * @return \string $image
	 */
	public function getImage() {
		return $this->image;
	}

	/**
	 * Sets the image
	 *
	 * @param \string $image
	 * @return void
	 */
	public function setImage($image) {
		$this->image = $image;
	}

	/**
	 * Returns the pkid
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Pkid $pkid
	 */
	public function getPkid() {
		return $this->pkid;
	}

	/**
	 * Sets the pkid
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Pkid $pkid
	 * @return void
	 */
	public function setPkid(\MM\MmDynbanner\Domain\Model\Pkid $pkid) {
		$this->pkid = $pkid;
	}

}
?>