<?php
namespace MM\MmDynbanner\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Campaign extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * Name
	 *
	 * @var \string
	 * @validate NotEmpty
	 */
	protected $name;

	/**
	 * Order Date
	 *
	 * @var \DateTime
	 */
	protected $orderDate;

	/**
	 * Client Name
	 *
	 * @var \string
	 */
	protected $clientName;

	/**
	 * Banners
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Banner>
	 * @lazy
	 */
	protected $banners;

	/**
	 * __construct
	 *
	 * @return Campaign
	 */
	public function __construct() {
		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all ObjectStorage properties.
	 *
	 * @return void
	 */
	protected function initStorageObjects() {
		/**
		 * Do not modify this method!
		 * It will be rewritten on each save in the extension builder
		 * You may modify the constructor of this class instead
		 */
		$this->banners = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Returns the name
	 *
	 * @return \string $name
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * Sets the name
	 *
	 * @param \string $name
	 * @return void
	 */
	public function setName($name) {
		$this->name = $name;
	}

	/**
	 * Returns the orderDate
	 *
	 * @return \DateTime $orderDate
	 */
	public function getOrderDate() {
		return $this->orderDate;
	}

	/**
	 * Sets the orderDate
	 *
	 * @param \DateTime $orderDate
	 * @return void
	 */
	public function setOrderDate($orderDate) {
		$this->orderDate = $orderDate;
	}

	/**
	 * Returns the clientName
	 *
	 * @return \string $clientName
	 */
	public function getClientName() {
		return $this->clientName;
	}

	/**
	 * Sets the clientName
	 *
	 * @param \string $clientName
	 * @return void
	 */
	public function setClientName($clientName) {
		$this->clientName = $clientName;
	}

	/**
	 * Adds a Banner
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $banner
	 * @return void
	 */
	public function addBanner(\MM\MmDynbanner\Domain\Model\Banner $banner) {
		$this->banners->attach($banner);
	}

	/**
	 * Removes a Banner
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $bannerToRemove The Banner to be removed
	 * @return void
	 */
	public function removeBanner(\MM\MmDynbanner\Domain\Model\Banner $bannerToRemove) {
		$this->banners->detach($bannerToRemove);
	}

	/**
	 * Returns the banners
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Banner> $banners
	 */
	public function getBanners() {
		return $this->banners;
	}

	/**
	 * Sets the banners
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Banner> $banners
	 * @return void
	 */
	public function setBanners(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $banners) {
		$this->banners = $banners;
	}

}
?>