<?php
namespace MM\MmDynbanner\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Rate extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * Name
	 *
	 * @var \string
	 * @validate NotEmpty
	 */
	protected $name;

	/**
	 * NameOrig
	 *
	 * @var \string
	 */
	protected $nameOrig;

	/**
	 * ShortName
	 *
	 * @var \string
	 */
	protected $shortName;

	/**
	 * Active
	 *
	 * @var boolean
	 */
	protected $active = FALSE;

	/**
	 * OV Month
	 *
	 * @var \integer
	 */
	protected $ovMonth;

	/**
	 * Legal
	 *
	 * @var \string
	 */
	protected $legal;
	
	/**
	 * Legal Texts (virtual Property)
	 *
	 * @var \array
	 */
	protected $legalTexts;

	/**
	 * Rate ID
	 *
	 * @var \string
	 */
	protected $rateId;

	/**
	 * Headline
	 *
	 * @var \string
	 */
	protected $headline;

	/**
	 * HC Size
	 *
	 * @var \integer
	 */
	protected $hcSize;

	/**
	 * Eyecatcher Stop
	 *
	 * @var \DateTime
	 */
	protected $eyecStop;

	/**
	 * Eyecatcher Text
	 *
	 * @var \string
	 */
	protected $eyecText;

	/**
	 * Phase1
	 *
	 * @var \float
	 */
	protected $phase1;

	/**
	 * Phase 2
	 *
	 * @var \float
	 */
	protected $phase2;

	/**
	 * Phase 3
	 *
	 * @var \float
	 */
	protected $phase3;

	/**
	 * Webtrekk
	 *
	 * @var \string
	 */
	protected $webtrekk;

	/**
	 * Cp17
	 *
	 * @var \string
	 */
	protected $cp17;

	/**
	 * Monthly price
	 *
	 * @var \float
	 */
	protected $monthlyPrice;

	/**
	 * Acitvation fee
	 *
	 * @var \float
	 */
	protected $activationFee;

	/**
	 * Archived
	 *
	 * @var boolean
	 */
	protected $archived = FALSE;

	/**
	 * isYoung
	 *
	 * @var boolean
	 */
	protected $isYoung = FALSE;

	/**
	 * TK Order
	 *
	 * @var \integer
	 */
	protected $tkOrder;

	/**
	 * Eyecatcher
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Eyecatcher
	 * @lazy
	 */
	protected $eyecatcher;

	/**
	 * Eyecatcher Color
	 *
	 * @var \MM\MmDynbanner\Domain\Model\EyecatcherColor
	 * @lazy
	 */
	protected $eyecatcherColor;

	/**
	 * PKID
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Pkid
	 * @lazy
	 */
	protected $pkid;

	/**
	 * Returns the name
	 *
	 * @return \string $name
	 */
	public function getName() {
		return $this->name;
	}

	/**
	 * Sets the name
	 *
	 * @param \string $name
	 * @return void
	 */
	public function setName($name) {
		$this->name = $name;
	}

	/**
	 * Returns the nameOrig
	 *
	 * @return \string $nameOrig
	 */
	public function getNameOrig() {
		return $this->nameOrig;
	}

	/**
	 * Sets the nameOrig
	 *
	 * @param \string $nameOrig
	 * @return void
	 */
	public function setNameOrig($nameOrig) {
		$this->nameOrig = $nameOrig;
	}

	/**
	 * Returns the shortName
	 *
	 * @return \string $shortName
	 */
	public function getShortName() {
		return $this->shortName;
	}

	/**
	 * Sets the shortName
	 *
	 * @param \string $shortName
	 * @return void
	 */
	public function setShortName($shortName) {
		$this->shortName = $shortName;
	}

	/**
	 * Returns the active
	 *
	 * @return boolean $active
	 */
	public function getActive() {
		return $this->active;
	}

	/**
	 * Sets the active
	 *
	 * @param boolean $active
	 * @return void
	 */
	public function setActive($active) {
		$this->active = $active;
	}

	/**
	 * Returns the boolean state of active
	 *
	 * @return boolean
	 */
	public function isActive() {
		return $this->getActive();
	}

	/**
	 * Returns the ovMonth
	 *
	 * @return \integer $ovMonth
	 */
	public function getOvMonth() {
		return $this->ovMonth;
	}

	/**
	 * Sets the ovMonth
	 *
	 * @param \integer $ovMonth
	 * @return void
	 */
	public function setOvMonth($ovMonth) {
		$this->ovMonth = $ovMonth;
	}

	/**
	 * Returns the legal
	 *
	 * @return \string $legal
	 */
	public function getLegal() {
		return $this->legal;
	}

	/**
	 * Sets the legal
	 *
	 * @param \string $legal
	 * @return void
	 */
	public function setLegal($legal) {
		$this->legal = $legal;
	}

	/**
	 * Returns the rateId
	 *
	 * @return \string $rateId
	 */
	public function getRateId() {
		return $this->rateId;
	}

	/**
	 * Sets the rateId
	 *
	 * @param \string $rateId
	 * @return void
	 */
	public function setRateId($rateId) {
		$this->rateId = $rateId;
	}

	/**
	 * Returns the headline
	 *
	 * @return \string $headline
	 */
	public function getHeadline() {
		return $this->headline;
	}

	/**
	 * Sets the headline
	 *
	 * @param \string $headline
	 * @return void
	 */
	public function setHeadline($headline) {
		$this->headline = $headline;
	}

	/**
	 * Returns the hcSize
	 *
	 * @return \integer $hcSize
	 */
	public function getHcSize() {
		return $this->hcSize;
	}

	/**
	 * Sets the hcSize
	 *
	 * @param \integer $hcSize
	 * @return void
	 */
	public function setHcSize($hcSize) {
		$this->hcSize = $hcSize;
	}

	/**
	 * Returns the eyecStop
	 *
	 * @return \DateTime $eyecStop
	 */
	public function getEyecStop() {
		return $this->eyecStop;
	}

	/**
	 * Sets the eyecStop
	 *
	 * @param \DateTime $eyecStop
	 * @return void
	 */
	public function setEyecStop($eyecStop) {
		$this->eyecStop = $eyecStop;
	}

	/**
	 * Returns the eyecText
	 *
	 * @return \string $eyecText
	 */
	public function getEyecText() {
		return $this->eyecText;
	}

	/**
	 * Sets the eyecText
	 *
	 * @param \string $eyecText
	 * @return void
	 */
	public function setEyecText($eyecText) {
		$this->eyecText = $eyecText;
	}

	/**
	 * Returns the phase1
	 *
	 * @return \float $phase1
	 */
	public function getPhase1() {
		return $this->phase1;
	}

	/**
	 * Sets the phase1
	 *
	 * @param \float $phase1
	 * @return void
	 */
	public function setPhase1($phase1) {
		$this->phase1 = $phase1;
	}

	/**
	 * Returns the phase2
	 *
	 * @return \float $phase2
	 */
	public function getPhase2() {
		return $this->phase2;
	}

	/**
	 * Sets the phase2
	 *
	 * @param \float $phase2
	 * @return void
	 */
	public function setPhase2($phase2) {
		$this->phase2 = $phase2;
	}

	/**
	 * Returns the phase3
	 *
	 * @return \float $phase3
	 */
	public function getPhase3() {
		return $this->phase3;
	}

	/**
	 * Sets the phase3
	 *
	 * @param \float $phase3
	 * @return void
	 */
	public function setPhase3($phase3) {
		$this->phase3 = $phase3;
	}

	/**
	 * Returns the webtrekk
	 *
	 * @return \string $webtrekk
	 */
	public function getWebtrekk() {
		return $this->webtrekk;
	}

	/**
	 * Sets the webtrekk
	 *
	 * @param \string $webtrekk
	 * @return void
	 */
	public function setWebtrekk($webtrekk) {
		$this->webtrekk = $webtrekk;
	}

	/**
	 * Returns the cp17
	 *
	 * @return \string $cp17
	 */
	public function getCp17() {
		return $this->cp17;
	}

	/**
	 * Sets the cp17
	 *
	 * @param \string $cp17
	 * @return void
	 */
	public function setCp17($cp17) {
		$this->cp17 = $cp17;
	}

	/**
	 * Returns the eyecatcher
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher
	 */
	public function getEyecatcher() {
		return $this->eyecatcher;
	}

	/**
	 * Sets the eyecatcher
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher
	 * @return void
	 */
	public function setEyecatcher(\MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher) {
		$this->eyecatcher = $eyecatcher;
	}


	/**
	 * Returns the eyecatcherColor
	 *
	 * @return \MM\MmDynbanner\Domain\Model\EyecatcherColor $eyecatcherColor
	 */
	public function getEyecatcherColor() {
		return $this->eyecatcherColor;
	}

	/**
	 * Sets the eyecatcherColor
	 *
	 * @param \MM\MmDynbanner\Domain\Model\EyecatcherColor $eyecatcherColor
	 * @return void
	 */
	public function setEyecatcherColor(\MM\MmDynbanner\Domain\Model\EyecatcherColor $eyecatcherColor =NULL) {
		$this->eyecatcherColor = $eyecatcherColor;
	}

	/**
	 * Returns the pkid
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Pkid $pkid
	 */
	public function getPkid() {
		return $this->pkid;
	}

	/**
	 * Sets the pkid
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Pkid $pkid
	 * @return void
	 */
	public function setPkid(\MM\MmDynbanner\Domain\Model\Pkid $pkid) {
		$this->pkid = $pkid;
	}

	/**
	 * Returns the monthlyPrice
	 *
	 * @return \float $monthlyPrice
	 */
	public function getMonthlyPrice() {
		return $this->monthlyPrice;
	}

	/**
	 * Sets the monthlyPrice
	 *
	 * @param \float $monthlyPrice
	 * @return void
	 */
	public function setMonthlyPrice($monthlyPrice) {
		$this->monthlyPrice = $monthlyPrice;
	}

	/**
	 * Returns the activationFee
	 *
	 * @return \float $activationFee
	 */
	public function getActivationFee() {
		return $this->activationFee;
	}

	/**
	 * Sets the activationFee
	 *
	 * @param \float $activationFee
	 * @return void
	 */
	public function setActivationFee($activationFee) {
		$this->activationFee = $activationFee;
	}
	
	
    /**
	 * Returns the legaTexts
	 *
	 * @return \array $legalTexts
	 */
	public function getLegalTexts() {
		return $this->legalTexts;
	}

	/**
	 * Sets the Lega Texts
	 *
	 * @param \array $legalTexts
	 * @return void
	 */
	public function setLegalTexts($legalTexts) {
		$this->legalTexts = $legalTexts;
	}

	/**
	 * Returns the Archived
	 *
	 * @return boolean $archived
	 */
	public function getArchived() {
		return $this->archived;
	}

	/**
	 * Sets the Archived
	 *
	 * @param boolean $archived
	 * @return void
	 */
	public function setArchived($archived) {
		$this->archived = $archived;
	}

	/**
	 * Returns the boolean state of Archived
	 *
	 * @return boolean
	 */
	public function isArchived() {
		return $this->getArchived();
	}

	/**
	 * Returns the IsYoung
	 *
	 * @return boolean $isYoung
	 */
	public function getIsYoung() {
		return $this->isYoung;
	}

	/**
	 * Sets the IsYoung
	 *
	 * @param boolean $archived
	 * @return void
	 */
	public function setIsYoung($isYoung) {
		$this->isYoung = $isYoung;
	}

	/**
	 * Returns the boolean state of Archived
	 *
	 * @return boolean
	 */
	public function isIsYoung() {
		return $this->getIsYoung();
	}

	/**
	 * Returns the tkOrder
	 *
	 * @return \integer $tkOrder
	 */
	public function getTkOrder() {
		return $this->tkOrder;
	}

	/**
	 * Sets the tkOrder
	 *
	 * @param \integer $tkOrder
	 * @return void
	 */
	public function setTkOrder($tkOrder) {
		$this->tkOrder = $tkOrder;
	}
	

}
