<?php
namespace MM\MmDynbanner\Domain\Repository;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class BannerRepository extends \TYPO3\CMS\Extbase\Persistence\Repository {

	/**
	 * @var array
	 */
    protected $defaultOrderings = array('rate' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING);

	/**
	* Injects the object manager
	*
	* @param \TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager
	* @return void
	*/
	public function injectObjectManager(\TYPO3\CMS\Extbase\Object\ObjectManagerInterface $objectManager) {
		$this->objectManager = $objectManager;
	}
 	
	public function initializeObject() {
			$querySettings = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\Typo3QuerySettings');
 	        $querySettings->setRespectStoragePage(FALSE);
 	        $this->setDefaultQuerySettings($querySettings);
	}

	public function findByFilter($filter) {
		$query = $this->createQuery();
		$query->matching (
			$query->like('productId',"%".$filter."%")
	  );
    return $query->execute();
	}

	public function findByFilterDev($filterdev) {
		
		$query = $this->createQuery();
		$query->matching (
			$query->like('device.nameOrig',"%".$filterdev."%")
	  );
    return $query->execute();
	}

	public function findByApprovedIdTest($approvedIdTest) {
		$rateList = array(69,70,71,72,73,74);
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->equals('approved_id',$approvedIdTest),
				$query->in('rate',$rateList)
				
			)		
	  );
    return $query->execute();
	}

	public function findByApprovedIdTest1($approvedIdTest) {
		$rateList = array(115,117,119);
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->equals('approved_id',$approvedIdTest),
				$query->in('rate',$rateList)
				
				
			)		
	  );
    return $query->execute();
	}
	  
	  // Search for approved banners in campaigns

	public function findByApprovedIdAndCampaign($approvedId,$campaignId) {
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->greaterThanOrEqual('approved_id',$approvedId),
				$query->equals('campaign',$campaignId),
				$query->equals('suspended', 0)
				
				
			)		
	  );
	  
	  
	  //\TYPO3\CMS\Extbase\Utility\DebuggerUtility::var_dump($tmpBanners);
    return $query->execute();
	}

	public function findAllByCampaign($campaignId) {
		$query = $this->createQuery();
		$query->matching (
			$query->logicalAnd(
				$query->equals('campaign',$campaignId)
			)		
	  );
	  
	  
	  //\TYPO3\CMS\Extbase\Utility\DebuggerUtility::var_dump($tmpBanners);
    return $query->execute();
	}

	public function findByApprovedIdAndCampaignAndRate($approvedId,$campaignId,$rateId) {
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->greaterThanOrEqual('approved_id',$approvedId),
				$query->equals('campaign',$campaignId),
                $query->equals('rate',$rateId)

				
				
			)		
	    );
	    return $query->execute();
	}

	public function findByApprovedIdAndCampaignSorted($approvedId,$campaignId,$sorter) {
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING,
			$sorter => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->greaterThanOrEqual('approved_id',$approvedId),
				$query->equals('campaign',$campaignId),
				$query->equals('suspended', 0)
				
				
			)		
	  );
	  return $query->execute();
	}
	
	public function findByApprovedIdAndCampaignAdform($approvedId,$campaignId) {
		$query = $this->createQuery();
		$query->setOrderings(array(
			'device' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_DESCENDING,
			'price' => \TYPO3\CMS\Extbase\Persistence\QueryInterface::ORDER_ASCENDING
		));
		$query->matching (
			$query->logicalAnd(
				$query->greaterThanOrEqual('approved_id',$approvedId),
				$query->equals('campaign',$campaignId),
				$query->equals('suspended', 0)
				
				
			)		
	  );
	  
	  
	  //\TYPO3\CMS\Extbase\Utility\DebuggerUtility::var_dump($tmpBanners);
    return $query->execute();
	}	
	
}
