<?php
namespace MM\MmDynbanner\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class OptionController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * optionRepository
	 *
	 * @var \MM\MmDynbanner\Domain\Repository\OptionRepository
	 * @inject
	 */
	protected $optionRepository;

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$options = $this->optionRepository->findAll();
		$this->view->assign('options', $options);
	}

	/**
	 * action show
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $option
	 * @return void
	 */
	public function showAction(\MM\MmDynbanner\Domain\Model\Option $option) {
		$this->view->assign('option', $option);
	}

	/**
	 * action new
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $newOption
	 * @dontvalidate $newOption
	 * @return void
	 */
	public function newAction(\MM\MmDynbanner\Domain\Model\Option $newOption = NULL) {
		if ($newOption == NULL) { // workaround for fluid bug ##5636
			$newOption = t3lib_div::makeInstance('');
		}
		$this->view->assign('newOption', $newOption);
	}

	/**
	 * action create
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $newOption
	 * @return void
	 */
	public function createAction(\MM\MmDynbanner\Domain\Model\Option $newOption) {
		$this->optionRepository->add($newOption);
		$this->flashMessageContainer->add('Your new Option was created.');
		$this->redirect('list');
	}

	/**
	 * action edit
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $option
	 * @return void
	 */
	public function editAction(\MM\MmDynbanner\Domain\Model\Option $option) {
		$this->view->assign('option', $option);
	}

	/**
	 * action update
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $option
	 * @return void
	 */
	public function updateAction(\MM\MmDynbanner\Domain\Model\Option $option) {
		$this->optionRepository->update($option);
		$this->flashMessageContainer->add('Your Option was updated.');
		$this->redirect('list');
	}

	/**
	 * action delete
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $option
	 * @return void
	 */
	public function deleteAction(\MM\MmDynbanner\Domain\Model\Option $option) {
		$this->optionRepository->remove($option);
		$this->flashMessageContainer->add('Your Option was removed.');
		$this->redirect('list');
	}

}
?>