<?php
namespace MM\MmDynbanner\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class BannerController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * bannerRepository
	 *
	 * @var \MM\MmDynbanner\Domain\Repository\BannerRepository
	 * @inject
	 */
	protected $bannerRepository;

	/**
	 * action list
	 *
	 * @return void
	 */
	public function listAction() {
		$banners = $this->bannerRepository->findAll();
		$this->view->assign('banners', $banners);
	}

	/**
	 * action show
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $banner
	 * @return void
	 */
	public function showAction(\MM\MmDynbanner\Domain\Model\Banner $banner) {
		$this->view->assign('banner', $banner);
	}

	/**
	 * action new
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $newBanner
	 * @dontvalidate $newBanner
	 * @return void
	 */
	public function newAction(\MM\MmDynbanner\Domain\Model\Banner $newBanner = NULL) {
		$this->view->assign('newBanner', $newBanner);
	}

	/**
	 * action create
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $newBanner
	 * @return void
	 */
	public function createAction(\MM\MmDynbanner\Domain\Model\Banner $newBanner) {
		$this->bannerRepository->add($newBanner);
		$this->flashMessageContainer->add('Your new Banner was created.');
		$this->redirect('list');
	}

	/**
	 * action edit
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $banner
	 * @return void
	 */
	public function editAction(\MM\MmDynbanner\Domain\Model\Banner $banner) {
		$this->view->assign('banner', $banner);
	}

	/**
	 * action update
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $banner
	 * @return void
	 */
	public function updateAction(\MM\MmDynbanner\Domain\Model\Banner $banner) {
		$this->bannerRepository->update($banner);
		$this->flashMessageContainer->add('Your Banner was updated.');
		$this->redirect('list');
	}

	/**
	 * action delete
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Banner $banner
	 * @return void
	 */
	public function deleteAction(\MM\MmDynbanner\Domain\Model\Banner $banner) {
		$this->bannerRepository->remove($banner);
		$this->flashMessageContainer->add('Your Banner was removed.');
		$this->redirect('list');
	}

}
?>