<?php
namespace MM\MmDynbanner\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Alexander Malicke <info@millemedia.de>, Millemedia
 *  
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 *
 *
 * @package mm_dynbanner
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class Banner extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity {

	/**
	 * Product Id
	 *
	 * @var \string
	 * @validate NotEmpty
	 */
	protected $productId;

	/**
	 * Headline
	 *
	 * @var \string
	 */
	protected $headline;

	/**
	 * HC Size
	 *
	 * @var \integer
	 */
	protected $hcSize;

	/**
	 * Subline
	 *
	 * @var \string
	 */
	protected $subline;

	/**
	 * Eyecatcher Start
	 *
	 * @var \DateTime
	 */
	protected $eyecStart;

	/**
	 * Eyecatcher Text
	 *
	 * @var \string
	 */
	protected $eyecText;

	/**
	 * Phase 1
	 *
	 * @var \float
	 */
	protected $phase1;

	/**
	 * Phase 2
	 *
	 * @var \float
	 */
	protected $phase2;

	/**
	 * Phase 3
	 *
	 * @var \float
	 */
	protected $phase3;

	/**
	 * Link
	 *
	 * @var \string
	 */
	protected $link;

	/**
	 * Webtrekk
	 *
	 * @var \string
	 */
	protected $webtrekk;

	/**
	 * Cp17
	 *
	 * @var \string
	 */
	protected $cp17;

	/**
	 * Approved ID
	 *
	 * @var \integer
	 */
	protected $approvedId;

	/**
	 * Campaign
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Campaign
	 * @lazy
	 */
	protected $campaign;

	/**
	 * Device
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Device
	 * @lazy
	 */
	protected $device;

	/**
	 * Rate
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Rate
	 * @lazy
	 */
	protected $rate;

	/**
	 * Supply
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Supply
	 * @lazy
	 */
	protected $supply;

	/**
	 * Option
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Option
	 * @lazy
	 */
	protected $options;

	/**
	 * Comments
	 *
	 * @var \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Comment>
	 * @lazy
	 */
	protected $comments;

	/**
	 * Eyecatcher
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Eyecatcher
	 * @lazy
	 */
	protected $eyecatcher;

	/**
	 * Format
	 *
	 * @var \MM\MmDynbanner\Domain\Model\Format
	 * @lazy
	 */
	protected $format;

	/**
	 * __construct
	 *
	 * @return Banner
	 */
	public function __construct() {
		//Do not remove the next line: It would break the functionality
		$this->initStorageObjects();
	}

	/**
	 * Initializes all ObjectStorage properties.
	 *
	 * @return void
	 */
	protected function initStorageObjects() {
		/**
		 * Do not modify this method!
		 * It will be rewritten on each save in the extension builder
		 * You may modify the constructor of this class instead
		 */
		$this->comments = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
	}

	/**
	 * Returns the productId
	 *
	 * @return \string $productId
	 */
	public function getProductId() {
		return $this->productId;
	}

	/**
	 * Sets the productId
	 *
	 * @param \string $productId
	 * @return void
	 */
	public function setProductId($productId) {
		$this->productId = $productId;
	}

	/**
	 * Returns the headline
	 *
	 * @return \string $headline
	 */
	public function getHeadline() {
		return $this->headline;
	}

	/**
	 * Sets the headline
	 *
	 * @param \string $headline
	 * @return void
	 */
	public function setHeadline($headline) {
		$this->headline = $headline;
	}

	/**
	 * Returns the hcSize
	 *
	 * @return \integer $hcSize
	 */
	public function getHcSize() {
		return $this->hcSize;
	}

	/**
	 * Sets the hcSize
	 *
	 * @param \integer $hcSize
	 * @return void
	 */
	public function setHcSize($hcSize) {
		$this->hcSize = $hcSize;
	}

	/**
	 * Returns the subline
	 *
	 * @return \string $subline
	 */
	public function getSubline() {
		return $this->subline;
	}

	/**
	 * Sets the subline
	 *
	 * @param \string $subline
	 * @return void
	 */
	public function setSubline($subline) {
		$this->subline = $subline;
	}

	/**
	 * Returns the eyecStart
	 *
	 * @return \DateTime $eyecStart
	 */
	public function getEyecStart() {
		return $this->eyecStart;
	}

	/**
	 * Sets the eyecStart
	 *
	 * @param \DateTime $eyecStart
	 * @return void
	 */
	public function setEyecStart($eyecStart) {
		$this->eyecStart = $eyecStart;
	}

	/**
	 * Returns the eyecText
	 *
	 * @return \string $eyecText
	 */
	public function getEyecText() {
		return $this->eyecText;
	}

	/**
	 * Sets the eyecText
	 *
	 * @param \string $eyecText
	 * @return void
	 */
	public function setEyecText($eyecText) {
		$this->eyecText = $eyecText;
	}

	/**
	 * Returns the phase1
	 *
	 * @return \float $phase1
	 */
	public function getPhase1() {
		return $this->phase1;
	}

	/**
	 * Sets the phase1
	 *
	 * @param \float $phase1
	 * @return void
	 */
	public function setPhase1($phase1) {
		$this->phase1 = $phase1;
	}

	/**
	 * Returns the phase2
	 *
	 * @return \float $phase2
	 */
	public function getPhase2() {
		return $this->phase2;
	}

	/**
	 * Sets the phase2
	 *
	 * @param \float $phase2
	 * @return void
	 */
	public function setPhase2($phase2) {
		$this->phase2 = $phase2;
	}

	/**
	 * Returns the phase3
	 *
	 * @return \float $phase3
	 */
	public function getPhase3() {
		return $this->phase3;
	}

	/**
	 * Sets the phase3
	 *
	 * @param \float $phase3
	 * @return void
	 */
	public function setPhase3($phase3) {
		$this->phase3 = $phase3;
	}

	/**
	 * Returns the link
	 *
	 * @return \string $link
	 */
	public function getLink() {
		return $this->link;
	}

	/**
	 * Sets the link
	 *
	 * @param \string $link
	 * @return void
	 */
	public function setLink($link) {
		$this->link = $link;
	}

	/**
	 * Returns the webtrekk
	 *
	 * @return \string $webtrekk
	 */
	public function getWebtrekk() {
		return $this->webtrekk;
	}

	/**
	 * Sets the webtrekk
	 *
	 * @param \string $webtrekk
	 * @return void
	 */
	public function setWebtrekk($webtrekk) {
		$this->webtrekk = $webtrekk;
	}

	/**
	 * Returns the cp17
	 *
	 * @return \string $cp17
	 */
	public function getCp17() {
		return $this->cp17;
	}

	/**
	 * Sets the cp17
	 *
	 * @param \string $cp17
	 * @return void
	 */
	public function setCp17($cp17) {
		$this->cp17 = $cp17;
	}

	/**
	 * Returns the approvedId
	 *
	 * @return \integer $approvedId
	 */
	public function getApprovedId() {
		return $this->approvedId;
	}

	/**
	 * Sets the approvedId
	 *
	 * @param \integer $approvedId
	 * @return void
	 */
	public function setApprovedId($approvedId) {
		$this->approvedId = $approvedId;
	}

	/**
	 * Returns the campaign
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Campaign $campaign
	 */
	public function getCampaign() {
		return $this->campaign;
	}

	/**
	 * Sets the campaign
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Campaign $campaign
	 * @return void
	 */
	public function setCampaign(\MM\MmDynbanner\Domain\Model\Campaign $campaign) {
		$this->campaign = $campaign;
	}

	/**
	 * Returns the device
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Device $device
	 */
	public function getDevice() {
		return $this->device;
	}

	/**
	 * Sets the device
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Device $device
	 * @return void
	 */
	public function setDevice(\MM\MmDynbanner\Domain\Model\Device $device) {
		$this->device = $device;
	}

	/**
	 * Returns the rate
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Rate $rate
	 */
	public function getRate() {
		return $this->rate;
	}

	/**
	 * Sets the rate
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Rate $rate
	 * @return void
	 */
	public function setRate(\MM\MmDynbanner\Domain\Model\Rate $rate) {
		$this->rate = $rate;
	}

	/**
	 * Returns the supply
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Supply $supply
	 */
	public function getSupply() {
		return $this->supply;
	}

	/**
	 * Sets the supply
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Supply $supply
	 * @return void
	 */
	public function setSupply(\MM\MmDynbanner\Domain\Model\Supply $supply) {
		$this->supply = $supply;
	}

	/**
	 * Returns the options
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Option $options
	 */
	public function getOptions() {
		return $this->options;
	}

	/**
	 * Sets the options
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Option $options
	 * @return void
	 */
	public function setOptions(\MM\MmDynbanner\Domain\Model\Option $options) {
		$this->options = $options;
	}

	/**
	 * Adds a Comment
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Comment $comment
	 * @return void
	 */
	public function addComment(\MM\MmDynbanner\Domain\Model\Comment $comment) {
		$this->comments->attach($comment);
	}

	/**
	 * Removes a Comment
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Comment $commentToRemove The Comment to be removed
	 * @return void
	 */
	public function removeComment(\MM\MmDynbanner\Domain\Model\Comment $commentToRemove) {
		$this->comments->detach($commentToRemove);
	}

	/**
	 * Returns the comments
	 *
	 * @return \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Comment> $comments
	 */
	public function getComments() {
		return $this->comments;
	}

	/**
	 * Sets the comments
	 *
	 * @param \TYPO3\CMS\Extbase\Persistence\ObjectStorage<\MM\MmDynbanner\Domain\Model\Comment> $comments
	 * @return void
	 */
	public function setComments(\TYPO3\CMS\Extbase\Persistence\ObjectStorage $comments) {
		$this->comments = $comments;
	}

	/**
	 * Returns the eyecatcher
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher
	 */
	public function getEyecatcher() {
		return $this->eyecatcher;
	}

	/**
	 * Sets the eyecatcher
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher
	 * @return void
	 */
	public function setEyecatcher(\MM\MmDynbanner\Domain\Model\Eyecatcher $eyecatcher) {
		$this->eyecatcher = $eyecatcher;
	}

	/**
	 * Returns the format
	 *
	 * @return \MM\MmDynbanner\Domain\Model\Format $format
	 */
	public function getFormat() {
		return $this->format;
	}

	/**
	 * Sets the format
	 *
	 * @param \MM\MmDynbanner\Domain\Model\Format $format
	 * @return void
	 */
	public function setFormat(\MM\MmDynbanner\Domain\Model\Format $format) {
		$this->format = $format;
	}

}
?>